package gov.anl.aps.dm.sync;

import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Date;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

public class PsqlConnection {
    Connection connection = null;
    private final Map<String, String> dmUsers = new HashMap<>();    // <username, username>
    private PreparedStatement insertQuery;
    private PreparedStatement updateQuery;
    private final String userTable = "user_info";
    private Date lastUpdate;

    private Logger logger;

    public PsqlConnection(Logger logger) {
    	this.logger = logger;
    }
    
    void connect(Properties config) throws SQLException {
        try {
            Class.forName("org.postgresql.Driver");
        } catch (ClassNotFoundException e) {
            logger.log(Level.SEVERE, "Can't establish PosgreSQL Driver", e);
            System.exit(0);
        }
        logger.log(Level.INFO, "PostgreSQL JDBC Driver Registered!");

        try {
            connection = DriverManager.getConnection(
                    config.getProperty("dm.database.connection"),
                    config.getProperty("dm.database.username"),
                    config.getProperty("dm.database.password"));
        } catch (SQLException e) {
            logger.log(Level.SEVERE, "Can't connect to PostgreSQL data base");
            throw e;
        }
    }
    
    void init() throws SQLException {
        Statement statement = null;
        try {
            statement = connection.createStatement();
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "can't create statement for postgreSQL connection");
            throw ex;
        }
        ResultSet results = null;
        try {
            if (statement != null) {
                results = statement.executeQuery("SELECT * FROM " + userTable + ";");
            }
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "can't execute SELECT query from " + userTable + " table");
            throw ex;
        }
        try {
            if (results != null) {
                while (results.next()) {
                    String username = results.getString("username");
                    dmUsers.put(username, username);
                }
                results.close();
            }
            if (statement != null) {
                statement.close();
            }
        } catch (SQLException ex) {
            logger.log(Level.WARNING, "query results processing error ", ex);
        }
       
        // prepare statements
        try {
            insertQuery = connection.prepareStatement ("INSERT INTO user_info (username, first_name, last_name, middle_name, email, badge) VALUES (?, ?, ?, ?, ?, ?)");
            updateQuery = connection.prepareStatement ("UPDATE user_info SET email = ?, last_name = ? WHERE username = ?");
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "Can't get peaparedStatement from connection ", ex);
            throw ex;
        }

        // get the last update
        String lastUpdateTable = "users_last_update";
        try {
            statement = connection.createStatement();
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "can't create statement for postgreSQL connection");
            throw ex;
        }
        results = null;
        try {
            if (statement != null) {
                results = statement.executeQuery("SELECT * FROM " + lastUpdateTable + " WHERE id = 1;");
            }
        } catch (SQLException ex) {
            logger.log(Level.WARNING, "can't execute query from {0} table", lastUpdateTable);
            addLastUpdate();
        }
        try {
            if (results != null) {
                results.next();
                lastUpdate = results.getDate("last_update");
                results.close();
            }
            if (statement != null) {
                statement.close();
            }
        } catch (SQLException ex) {
            logger.log(Level.WARNING, "The table {0} is empty", lastUpdateTable);
            addLastUpdate();
        }       
    }
    
    boolean isUser(String username) {
        return dmUsers.containsKey(username);
    }
    
    Date getLastUpdate() {
        return lastUpdate;
    }

    void addLastUpdate() {
        java.util.Calendar cal = java.util.Calendar.getInstance();
        cal.set(1990, Calendar.JANUARY, 1);
        try {
            lastUpdate = new Date(cal.getTimeInMillis());
            try (
                    PreparedStatement setUpdate = connection.prepareStatement("INSERT INTO users_last_update (id, last_update) VALUES (1, ?)")) {
                    setUpdate.setDate(1, lastUpdate);
                    setUpdate.executeUpdate();
            }
        } catch (SQLException ex) {
            logger.log(Level.WARNING, "can't update last update table ", ex);
        }
    }

    void setLastUpdate() {
        java.util.Calendar cal = java.util.Calendar.getInstance();
        try {
            lastUpdate = new Date(cal.getTimeInMillis());
            try (
                    PreparedStatement setUpdate = connection.prepareStatement("UPDATE users_last_update SET last_update = ?")) {
                    setUpdate.setDate(1, lastUpdate);
                    setUpdate.execute();
            }
        } catch (SQLException ex) {
            logger.log(Level.WARNING, "can't update last update table ", ex);
        }
    }

    void addUser(DmUser dmuser) {
        try {
            insertQuery.setString(1, dmuser.username);
            insertQuery.setString(2, dmuser.firstName);
            insertQuery.setString(3, dmuser.lastName);
            insertQuery.setString(4, dmuser.middleName);
            insertQuery.setString(5, dmuser.email);
            insertQuery.setString(6, dmuser.badge);
            insertQuery.execute();
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "can't execute query to add user ", ex);
        }
    }
 
    void updateUser(DmUser dmuser) {
        try {
            updateQuery.setString(1, dmuser.email);
            updateQuery.setString(2, dmuser.lastName);
            updateQuery.setString(3, dmuser.username);
            updateQuery.executeUpdate();
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "can't execute query to update user", ex);
        }
    }
 
    void close() {
        try {
            insertQuery.close();
            updateQuery.close();
            connection.close();
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "problem closing PostgreSQL connection ", ex);
        }
    }
}
